@@ -1,37 +1,36 @@
 #ifndef foodaemonforkhfoo
 #define foodaemonforkhfoo
 
-/* $Id: dfork.h 116 2007-06-11 15:16:09Z lennart $ */
+/***
+  This file is part of libdaemon.
 
-/*
- * This file is part of libdaemon.
- *
- * libdaemon is free software; you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as
- * published by the Free Software Foundation; either version 2.1 of
- * the License, or (at your option) any later version.
- *
- * libdaemon is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with libdaemon; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
- * 02110-1301 USA
- */
+  Copyright 2003-2008 Lennart Poettering
+
+  libdaemon is free software; you can redistribute it and/or modify
+  it under the terms of the GNU Lesser General Public License as
+  published by the Free Software Foundation, either version 2.1 of the
+  License, or (at your option) any later version.
+
+  libdaemon is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+  Lesser General Public License for more details.
+
+  You should have received a copy of the GNU Lesser General Public
+  License along with libdaemon. If not, see
+  <http://www.gnu.org/licenses/>.
+***/
 
 #include <sys/types.h>
 
 #ifdef __cplusplus
 extern "C" {
 #endif
-    
+
 /** \mainpage libdaemon
  *
  * libdaemon
- * 
+ *
  * For a brief explanation of libdaemons's purpose, have a look on the
  * README file. Thank you!
  *
@@ -43,7 +42,7 @@ extern "C" {
 
 /** \file
  *
- * Contains an API for doing a daemonizing fork(). 
+ * Contains an API for doing a daemonizing fork().
  *
  * You may daemonize by calling daemon_fork(), a function similar to
  * the plain fork(). If you want to return a return value of the
@@ -100,13 +99,39 @@ int daemon_retval_wait(int timeout);
  */
 int daemon_retval_send(int s);
 
+/** This variable is defined to 1 iff daemon_close_all() and daemon_close_allv() are supported.*/
+#define DAEMON_CLOSE_ALL_AVAILABLE 1
+
 /** Close all file descriptors except those passed. List needs to be
  * terminated by -1. FDs 0, 1, 2 will be kept open anyway. */
 int daemon_close_all(int except_fd, ...);
 
-/** Same as daemon_close_all but takes an array of fds, terminated by -1 */
+/** Same as daemon_close_all but takes an array of fds, terminated by
+ * -1 */
 int daemon_close_allv(const int except_fds[]);
 
+/** This variable is defined to 1 iff daemon_unblock_sigs() and daemon_unblock_sigsv() are supported.*/
+#define DAEMON_UNBLOCK_SIGS_AVAILABLE 1
+
+/** Unblock all signals except those passed. List needs to be
+ * terminated by -1. */
+int daemon_unblock_sigs(int except, ...);
+
+/** Same as daemon_unblock_sigs() but takes an array of signals,
+ * terminated by -1 */
+int daemon_unblock_sigsv(const int except[]);
+
+/** This variable is defined to 1 iff daemon_reset_sigs() and daemon_reset_sigsv() are supported.*/
+#define DAEMON_RESET_SIGS_AVAILABLE 1
+
+/** Reset all signal handlers except those passed. List needs to be
+ * terminated by -1. */
+int daemon_reset_sigs(int except, ...);
+
+/** Same as daemon_reset_sigs() but takes an array of signals,
+ * terminated by -1 */
+int daemon_reset_sigsv(const int except[]);
+
 #ifdef __cplusplus
 }
 #endif
